<?php
/**
 * @file class/sourceGuard.class.php
 * @author Mateusz Mania
 * @version 1.0
 */

/**
 * Main class.
 */
class sourceGuard{
	
/**
 * SourceGuard class constructor.
 */
	function __construct(){
		
		
	}
	
/**
 * Run a SourceGuard script.
 * Example:
 * <code>
 * include('path/to/sourceGuard.class.php');
 * 
 * $sourceGuard = new SourceGuard(); //Object declaration
 * 
 * echo '<b>Hello World</b>'; //Your code
 * 
 * $sourceGuard -> Run(1); //Run script
 * </code>
 * @param int $mode - encoding level (1 - HTML obfuscation, 2 - JavaScript encoding, 3 - HTML obfuscation + JavaScript encoding)
 */
	public function Run($c, $mode){
		
		$output = new sourceGuardOutput($c);
		
		$output -> Encode($mode);
		
		//ob_end_clean();
		
		return $output -> content;
	}
}

/**
 * Output class.
 */
class sourceGuardOutput extends sourceGuard{
	
/**
 * SourceGuardOutput class constructor.
 */
	function __construct($c){
		
		$this -> content = $c;
	}
	
	protected function Encode($mode){
		
		if($mode < 1 || $mode > 3){
			
			$mode = 3;
		}
		
		if(($mode == 1) || ($mode == 3)){
			
			HTMLObfuscator::Obfuscate(); 
		}
		
		if(($mode == 2) || ($mode == 3)){
			
			JSEncoder::Encode();
		}
	}
}

/**
 * HTMLObfuscator class.
 */
abstract class HTMLObfuscator extends sourceGuardOutput{
	
	protected function Obfuscate(){
		
		//removing popular white spaces
		$this -> content = str_replace(array("\r", "\n", "	"), '', $this -> content);
		
		$this -> content = preg_replace(
		array(
			
			"/\[&\]([^[]+)\[\/;\]/e",
			"/&([a-zA-Z0-9#]+);/e",
			"/ (align|alt|border|cellpadding|cellspacing|class|cols|colspan|for|frameborder|height|href|id|label|method|name|readonly|rows|rowspan|selected|src|target|title|type|value|width)=\"([^\&\"]+)\"/e",
			"/<(a|b|button|center|div|em|fieldset|font|form|h1|h2|h3|h4|h5|h6|i|noscript|ol|optgroup|option|p|small|span|strong|sub|sup|td|textarea|th)([^<]*)>([^<>&]+)<([\/]?)\\1/e",
			"/\[&\]([^[]+)\[\/;\]/e"
		),
		array(
			
			"htmlspecialchars('[&]$1[/;]')",
			"eval('return \'[&]$1[/;]\';');",
			"eval('\$r = HTMLObfuscator::Replace(\"$2\"); return \" $1=\\\"\".\$r.\"\\\"\";')",
			"eval('\$r=HTMLObfuscator::Replace(\"$3\"); return \"<$1$2>\".\$r.\"<$4$1\";')",
			"eval('return \"&\".preg_replace(\"/&#([a-z0-9]+);/e\", \"chr(\'\\$1\')\", \"$1\").\";\";')"
		),
		$this -> content
		);
	}
	
	private function Replace($input){
		
			foreach(str_split($input) AS $index => $char){
				
				$ord = ord($char);
				
				if((($ord > 32) && ($ord < 94)) || (($ord > 96) && ($ord < 123))){
					
					$r.= "&#".$ord.";";
				}
				else{
					
					$r.= $char;
				}
			}
			
			return $r;
	}
}

/**
 * JSObfuscator class.
 */
abstract class JSEncoder extends sourceGuardOutput{
	
/**
 * Method transform output to JS encoding function.
 */
	protected function Encode(){
		
		$a = array('q', 'w', 'e', 'r', 't', 'y', 'u', 'i', 'o', 'p', '[', ']', 'a', 's', 'd', 'f', 'g', 'h', 'j', 'k', 'l', 'z', 'x', 'c', 'v', 'b', 'n', 'm', '-', '~', '!', '@', '*', '(', ')', '_', '+', 'Q', 'W', 'E', 'R', 'T', 'Y', 'U', 'I', 'O', 'P', '{', '}', 'A', 'S', 'D', 'F', 'G', 'H', 'J', 'K', 'L', 'Z', 'X', 'C', 'V', 'B', 'N', 'M', '<', '>', '=');
		
		shuffle($a);
		
		foreach($a AS $index => $key){
			
			$b[$index] = '&'.$index.';';
		}
		
		
		$encodedString = str_replace($a, $b, $this -> content);
		
		
		for($i=0;$i<5;$i++){
			
			$rs = ((($i+1)*100)+1);
			$rl = ($rs+99);
			$v[$i] = 'a'.md5(rand($rs, $rl));
		}
		
		//ie fix
		preg_match("/^<!doctype([^>]+)>/i", $this -> content, $doctype_exp);
		
		$this -> content = $doctype_exp[1];
		
		
		$this -> content = $doctype_exp[0].'<script>'.JSEncoder::Escape('unescape=function ('.$v[0].','.$v[1].','.$v[2].'){'.$v[3].'='.$v[0].';for('.$v[4].'=0;'.$v[4].'<'.$v[1].'.length;'.$v[4].'++){'.$v[3].'='.$v[3].'.replace(new RegExp('.$v[1].'['.$v[4].'],"g"),'.$v[2].'['.$v[4].']);} '.$v[3].'='.$v[3].'.replace(new RegExp("%26", "g"), "&"); '.$v[3].'='.$v[3].'.replace(new RegExp("%3B", "g"), ";"); document.write('.$v[3].'.replace(\'<!--?--><?\',\'<!--?-->\')); '.$v[0].'="";'.$v[1].'="";'.$v[2].'="";'.$v[3].'="";}').' eval(unescape('.json_encode($encodedString).','.json_encode($b).','.json_encode($a).'));</script>';
	}
	
/**
 * @param string $str
 * @return string
 */
	private function Escape($str){
		
		$str_split = str_split($str);
		
		foreach($str_split AS $index => $char){
			
			$output .= '%'.strtoupper(bin2hex($char)).'';
		}
		
		return 'eval(unescape("'.$output.'"));';
	}
}
?>